From 1825f26399c3f44b012bfcb09ff2f1b5dd610259 Mon Sep 17 00:00:00 2001
From: Takashi Yano <takashi.yano@nifty.ne.jp>
Date: Wed, 15 Sep 2021 08:50:52 +0900
Subject: [PATCH] Cygwin: pipe: Do not call PeekNamedPipe() if it is not
 necessary.

- In pipe_data_available() in select.cc, PeekNamedPipe() call is
  not needed if WriteQuotaAvailable is non-zero because we already
  know the write pipe has a space. Therefore, with this patch,
  PeekNamedPipe() is called only when WriteQuotaAvailable is zero.
  This makes select() on pipe faster a bit.
---
 winsup/cygwin/select.cc | 6 +++---
 1 file changed, 3 insertions(+), 3 deletions(-)

diff --git a/winsup/cygwin/select.cc b/winsup/cygwin/select.cc
index ac2f3a9e0..8633e88ac 100644
--- a/winsup/cygwin/select.cc
+++ b/winsup/cygwin/select.cc
@@ -636,14 +636,14 @@ pipe_data_available (int fd, fhandler_base *fh, HANDLE h, bool writing)
       if (fh->get_device () == FH_PIPEW)
 	{
 	  HANDLE query_hdl = ((fhandler_pipe *) fh)->get_query_handle ();
-	  if (query_hdl)
+	  if (!query_hdl)
+	    return 1; /* We cannot know actual write pipe space. */
+	  else if (fpli.WriteQuotaAvailable == 0)
 	    {
 	      DWORD nbytes_in_pipe;
 	      PeekNamedPipe (query_hdl, NULL, 0, NULL, &nbytes_in_pipe, NULL);
 	      fpli.WriteQuotaAvailable = fpli.InboundQuota - nbytes_in_pipe;
 	    }
-	  else
-	    return 1;
 	}
       if (fpli.WriteQuotaAvailable > 0)
 	{
-- 
2.33.0

